% Lausanne, 9/1/2017
% -------------------------------------------------------------------
% demoSWMTextureLearning2D.m
% 
% Demo script for SWM texture learning
% IMPORTANT: It requires compiling the libsvm wrapper for matlab: 
% execute onces make.m in ./SteerableWaveletMachines/libsvm/matlab/
% -------------------------------------------------------------------
%
% Adrien Depeursinge, adrien.depeursinge@epfl.ch
% EPFL, Biomedical Imaging Group (BIG), Lausanne, Switzerland
% HES-SO Valais, MedGIFT, Institute of Information Systems, Sierre, Switzerland
% -------------------------------------------------------------------
%
% REFERENCES:
% [1] A. Depeursinge, Z. Puspoki, J.P. Ward, M. Unser,
%       "Steerable Wavelet Machines (SWM): Learning Moving Frames for
%       Texture Classification", in press.
%
% [2] M. Unser and N. Chenouard, "A Unifying Parametric Framework for 2D Steerable
%       Wavelet Transforms", SIAM J. Imaging Sci., vol. 6., pp 102-135, 2013.
%
% -------------------------------------------------------------------

clearvars; close all; clc;

% Create toy images I1 and I2
imageSize = 128;
[X,Y] = meshgrid(1:imageSize,1:imageSize);
stitchScale = 96;
I1 = sin(X/(imageSize/stitchScale))+sin(Y/(imageSize/stitchScale)); % checkerboard
I2 = sin(X/(imageSize/stitchScale)); % vertical lines

subplot(3,4,1); imshow(I1,[]); title('texture1');
subplot(3,4,5); imshow(I2,[]); title('texture2');

% parameters for texture analysis
harmonicsVector = 0:1:5                              % number of circular harmonics
num_scales = 4;                                      % number of scales: number of iterations of the isotropic wavelet transform
pyramid = 0;                                         % 1: decimated, 0: undecimated wavelet transform
cropSupport = 1;                                     % crop the mask based on the spatial support of the wavelet to avoid the influence of surrounding object

% extract circular harmonic wavelet (CHW) coefficients
Qu1 = SWMtextureAnalysis(I1,harmonicsVector,num_scales,pyramid,0);
Qu2 = SWMtextureAnalysis(I2,harmonicsVector,num_scales,pyramid,0);

% texture features: compute mean of the abs of the CHW within an image region defined by a binary mask
mask = ones(size(I1));
features1 = averageChannelAbsInMask(Qu1,mask,'real',harmonicsVector,pyramid,cropSupport);
features2 = averageChannelAbsInMask(Qu2,mask,'real',harmonicsVector,pyramid,cropSupport);

% learn feature weights
[weightsAlign,~] = buildLinearSVMs(features1,features2);

% plot features and learned weights
subplot(3,4,[2,3,4]);
bar(features1);
set(gca, 'XTick', 1:size(features1,2));
set(gca, 'XTickLabel', harmonicsVector);
xlabel('harmonics and scales');
title('features1');
subplot(3,4,[6,7,8]);
bar(features2);
set(gca, 'XTick', 1:size(features1,2));
set(gca, 'XTickLabel', harmonicsVector);
xlabel('harmonics and scales');
title('features2');
subplot(3,4,[10,11,12]);
bar(weightsAlign);
set(gca, 'XTick', 1:size(features1,2));
set(gca, 'XTickLabel', harmonicsVector);
xlabel('harmonics and scales');
title('learned weights');

% align CHW coefficients based on the learned weights
Qu1 = SWMtextureAnalysis(I1,harmonicsVector,num_scales,pyramid,1,weightsAlign);
Qu2 = SWMtextureAnalysis(I2,harmonicsVector,num_scales,pyramid,1,weightsAlign);

set(gcf, 'units','normalized','outerposition',[0 0 1 1]); % full screen
