% Lausanne, 9/1/2017
% -------------------------------------------------------------------
% demoSWMTextureSegmentation2D.m
% 
% Demo script for SWM texture segmentation
% IMPORTANT: It requires compiling the libsvm wrapper for matlab: 
% execute onces make.m in ./SteerableWaveletMachines/libsvm/matlab/
% -------------------------------------------------------------------
%
% Adrien Depeursinge, adrien.depeursinge@epfl.ch
% EPFL, Biomedical Imaging Group (BIG), Lausanne, Switzerland
% HES-SO Valais, MedGIFT, Institute of Information Systems, Sierre, Switzerland
% -------------------------------------------------------------------
%
% REFERENCES:
% [1] A. Depeursinge, Z. Puspoki, J.P. Ward, M. Unser,
%       "Steerable Wavelet Machines (SWM): Learning Moving Frames for
%       Texture Classification", in press.
%
% [2] M. Unser and N. Chenouard, "A Unifying Parametric Framework for 2D Steerable
%       Wavelet Transforms", SIAM J. Imaging Sci., vol. 6., pp 102-135, 2013.
%
% -------------------------------------------------------------------

clearvars;close all;clc;

% train images
I1 = imread('000180.ras'); % Outex 2) canvas002
I2 = imread('000360.ras'); % Outex 3) canvas003

% create test image 
A = imread('000247.ras'); % Outex 2) canvas002
B = imread('000423.ras'); % Outex 3) canvas003
C = imread('000510.ras'); % Outex 3) canvas003
D = imread('000335.ras'); % Outex 2) canvas002
testImage = [A B;C D];

truth = [ones(size(A)) -ones(size(B));-ones(size(C)) ones(size(D))]; 

figure;
subplot(3,3,1); imshow(I1,[]); title('train class 1');
subplot(3,3,2); imshow(I2,[]); title('train class 2');
subplot(3,3,4); imshow(testImage,[]); title('test image');

% parameters for texture analysis
patchRadius = 32;                                                 % radius of the circular patch
distCtrs = 16;                                                    % distance between the centers of the patches (controls patch overlap)
harmonicsVector = 0;                                            % number of circular harmonics
num_scales = 6;                                                   % number of scales: number of iterations of the isotropic wavelet transform
pyramid = 0;                                                      % 1: decimated, 0: undecimated wavelet transform
align = 0;                                                        % 0: no steering, >1: moving frames built from scale = align
complexType = 'abs';                                              % 'abs' is recommended if align = 0. 'concatenated' should be used otherwise
cropSupport = 0;                                                  % crop the mask based on the spatial support of the wavelet to avoid the influence of surrounding objects

% extract circular harmonic wavelet (CHW) coefficients
Qu1 = SWMtextureAnalysis(I1,harmonicsVector,num_scales,pyramid,align);
Qu2 = SWMtextureAnalysis(I2,harmonicsVector,num_scales,pyramid,align);
Qu  = SWMtextureAnalysis(testImage,harmonicsVector,num_scales,pyramid,align);

% patch-based analysis
features1 = [];
features2 = [];
for x=1:distCtrs:size(I1,1)+distCtrs
    for y=1:distCtrs:size(I1,2)+distCtrs
        
        patch = getCircularPatch(I1,x,y,patchRadius);
        
%         figure;imshow(patch,[]);
        feat1 = averageChannelAbsInMask(Qu1,patch,complexType,harmonicsVector,pyramid,cropSupport);
        feat2 = averageChannelAbsInMask(Qu2,patch,complexType,harmonicsVector,pyramid,cropSupport);

        features1 = [features1;feat1];
        features2 = [features2;feat2];
    end;
end;
[~,model,minFeat,maxFeat] = buildLinearSVMs(features1,features2);

% classify each pixel
decValues = zeros(size(testImage));
halfDistCtrs = floor(distCtrs/2);
for x=1:distCtrs:size(testImage,1)+distCtrs
    for y=1:distCtrs:size(testImage,2)+distCtrs
        
        % compute features in patch
        patch = getCircularPatch(testImage,x,y,patchRadius);
        feat = averageChannelAbsInMask(Qu,patch,complexType,harmonicsVector,pyramid,cropSupport);
        
        % normalize features in [0,1]
        feat = feat-repmat(minFeat,[size(feat,1) 1]);
        feat = feat./repmat(maxFeat,[size(feat,1) 1]);
        
        % fill decision value map
        xRange = max([x-halfDistCtrs 1]):min([x+halfDistCtrs size(testImage,1)]);
        yRange = max([y-halfDistCtrs 1]):min([y+halfDistCtrs size(testImage,2)]);
        [~,~,decValues(xRange,yRange)] = svmpredict(1,feat,model,'-q');        
    end;
end;

decValues = decValues - mean(decValues(:));
predictedLabels = sign(decValues);
errorPred = abs(predictedLabels - truth)/2;

patchExample = getCircularPatch(testImage,patchRadius+10,patchRadius+10,patchRadius);
subplot(3,3,5);
imshow(patchExample,[]); title(['patch radius=' num2str(patchRadius)]);
subplot(3,3,7);
imshow(decValues,[]); title('decision values');
subplot(3,3,8);
imshow(predictedLabels,[]); title('predicted labels');
subplot(3,3,9);
imshow(errorPred,[]); title(['error=' num2str(sum(errorPred(:))/(size(errorPred,1)*size(errorPred,2)))]);

set(gcf, 'units','normalized','outerposition',[0 0 1 1]); % full screen
